#  LCA Assignment 3 - Maintenance Planning of Integrated Systems and Multi-Objective Optimization
#     Group G:
#     Mohamed Abdelmotaal  - 494293
#     Dani    Zaeem        - 480307
#     Niko Hila            - 379143
#     Usama Ali Khan       - 494296
#     Alireza Solhjou      - 494248
#     Ioannis Papatsoutsos - 398734


rm (list = ls())
#Initialization
library(timelineS) 
library(dplyr) 
library(ggplot2) 
library(reshape2) 
library(lubridate)
library(rPref)
library(mco)
library(MASS)
library(shiny)
library(readr)
library(scales)
library(reshape2)
library(tibble)

plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) + 
                        years(lifetime))
  timelineS(dataP, main = plot.Name,labels = events.name, label.direction = "up", label.position = 3)
}
dist.Events <- function (lifetime, events, start.Date, option.Name) {
  
  events <- sort(events, decreasing = T)
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  all.events <- melt(distribution.events)
  #all events unlisted
  all.events <- melt(distribution.events)
  
  if (length(dim(all.events)) == 2 && ncol(all.events) == 2) {
    colnames(all.events) <- c("frequency", "event")
  } else {
    all.events <- data.frame(frequency = rep(0, lifetime + 1), event="DC")
  }
  all.events <- all.events[order(all.events$frequency),]
  unique.events <- all.events[!duplicated(all.events$frequency),]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
  #plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name)
  return(unique.events)
}


combine.lifeTimelines <- function(product.1, product.2, p1.dur, p2.dur) {
  
  base <- list(Names = c(), frequency = c(), duration = c())
  
  base$frequency <- sort(unique(c(product.1$frequency,product.2$frequency)),decreasing = FALSE)
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  
  base$duration[length(base$frequency)] <- 0
'  for(index in 2:(length(base$frequency) - 1)) {
    
    phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])]
    phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])]
    

    if(length(phase.1) != 0 && length(phase.2) != 0) {
      base$Names[index] <- paste(phase.1, phase.2)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p2.dur[which(names(p2.dur) == phase.2)])
    } else if(length(phase.1) != 0 && length(phase.2) == 0) {
      base$Names[index] <- phase.1
      base$duration[index] <- p1.dur[which(names(p1.dur) == phase.1)]
    } else if (length(phase.1) == 0 && length(phase.2) != 0) {
      base$Names[index] <- phase.2
      base$duration[index] <- p2.dur[which(names(p2.dur) == phase.2)]
    } else {
      base$duration[index] <- 0
    }
  }
  return(base)
}'
  for(index in 2:(length(base$frequency) - 1)) {
    phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])]
    phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])]
    
    if (length(phase.1) != 0) {
      p1_index <- which(names(p1.dur) == phase.1)
      if (length(p1_index) != 0) {
        base$duration[index] <- p1.dur[p1_index]
      } else {
        base$duration[index] <- 0
      }
    } else {
      base$duration[index] <- 0
    }
    
    if (length(phase.2) != 0) {
      p2_index <- which(names(p2.dur) == phase.2)
      if (length(p2_index) != 0) {
        base$duration[index] <- max(base$duration[index], p2.dur[p2_index])
      } else {
        base$duration[index] <- max(base$duration[index], 0)
      }
    } else {
      base$duration[index] <- max(base$duration[index], 0)
    }
  }
  return(base)
}  
  
par(mfrow = c(2, 1)) #split the plot in 2 rows and 1 column
start.Date = "2024-01-01"
lifetime <- 90

#First strategy of maintenance:
events.Op1.Factory <- c(IC = 3, PR = 36, TR = 50, S = 27, G = 18, RS = 36,  END = lifetime) #interval between interventions in Years
duration.ev.Factory <- c(IC = 1, PR = 3, TR = 7, S = 2, G = 1, RS = 3) #duration of interventions in Days
events.Op1.Railway_System <- c(M = 1, I = 6, RC= 50, RW = 80, RPR = 8, R = 40, END = lifetime) 
duration.ev.Railway_System <- c(M = 2, I = 1, RC = 3, RW = 5, RPR = 7, R = 15)

design.Options.Factory <- list(design.Op1.Factory <- "Main-Structure - Factory")
design.Options.Railway_System <- list(design.Op1.Railway_System <- "Railway_System")

maintenance.Factory <- dist.Events(lifetime, events.Op1.Factory, start.Date, design.Options.Factory)
maintenance.Railway_System <- dist.Events(lifetime, events.Op1.Railway_System, start.Date, design.Options.Railway_System)
plot.timeline(maintenance.Factory$frequency, maintenance.Factory$event, start.Date, design.Options.Factory)
plot.timeline(maintenance.Railway_System$frequency, maintenance.Railway_System$event, start.Date, design.Options.Railway_System)

integrated.interv <- combine.lifeTimelines(maintenance.Factory, maintenance.Railway_System, duration.ev.Factory, duration.ev.Railway_System)
sum(integrated.interv$duration) # total duration of the interruptions for this maintenance strategy

#Second strategy of maintenance: 
events.Op1.Factory <- c(IC = 3, RP = 5, TR = 50, S = 25, G = 15, RS = 36,  END = lifetime) #interval between interventions in Years
duration.ev.Factory <- c(IC = 1, RP = 3, TR = 7, S = 2, G = 1, RS = 3) #duration of interventions in Days
events.Op1.Railway_System <- c(M = 1, I = 6, RC = 50, RPR = 10, R = 40, END = lifetime) 
duration.ev.Railway_System <- c(M = 2, I = 1, RC = 4, RPR = 7, R = 15)

design.Options.Factory <- list(design.Op1.Factory <- "Main-Structure")
design.Options.Railway_System <- list(design.Op1.Railway_System <- "Railway_System")

maintenance.Factory <- dist.Events(lifetime, events.Op1.Factory, start.Date, design.Options.Factory)
maintenance.Railway_System <- dist.Events(lifetime, events.Op1.Railway_System, start.Date, design.Options.Railway_System)

integrated.interv <- combine.lifeTimelines(maintenance.Factory, maintenance.Railway_System, duration.ev.Factory, duration.ev.Railway_System)
sum(integrated.interv$duration) # total duration of the interruptions for this maintenance strategy

getmode <- function(v) {
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

#Function to automate the search for the best maintenance strategy 
design.explore <- function(events1, events2) {
  results <- c()
  for(i in 1: dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, design.Op1.Factory)
    dur.ev1 <- ev1/2
    for (j in 1: dim(events1)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, design.Op1.Railway_System)
      dur.ev2 <- ev2/2
      combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dur.ev1, dur.ev2)
      min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] -
                                combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
      results <- rbind(results, c(ev1, ev2,  dur = sum(combined.lifetime$duration), dist.inter = min.dist.int))
    }
  }
  return(as.data.frame(results))
}

#Generating the scenarios matrix (event1 and event2 in the function design.explore)
n.grid <- 2
events.grid.Factory <- expand.grid(IC = sample(seq(1,5, by = 1), n.grid),
                                   PR = sample(seq(36,40,1), n.grid),
                                   TR = sample(seq(40, 60, 1), n.grid),
                                   S = sample(seq(20, 30, 1), n.grid),
                                   G = sample(seq(10, 20, 1), n.grid),
                                   RS = sample(seq(30, 40, 1), n.grid),
                                   RP = sample(seq(1, 10, 1), n.grid))
events.grid.Railway_System <- expand.grid(M = sample(seq(1, 5, by = 1), n.grid),
                                          RPR = sample(seq(5,15,1), n.grid),
                                          I = sample(seq(1,10,1), n.grid),
                                          RC = sample(seq(45,55,1), n.grid),
                                          RW = sample(seq(75,85,1), n.grid),
                                          R = sample(seq(30, 50, 1), n.grid))
response.space <- design.explore(events.grid.Factory, events.grid.Railway_System)
p <- low(dur)* high(dist.inter)
sky <- psel(response.space, p)


pareto2 <- psel(response.space, p, top = nrow(response.space))
ggplot(response.space, aes(x = dur, y = dist.inter)) +
  geom_point(shape = 21) +
  geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))
show_front <- function(pref) {
  plot(response.space$dur, response.space$dist.inter)
  sky <- psel(response.space, pref)
  plot_front(response.space, pref, col = rgb(0, 0, 1))
  points(sky$dur, sky$dist.inter, lwd = 3)
}
show_front(p)
p <- high(dur) * low(dist.inter)
show_front(p)



LCI.materials <- read.csv("LCA3-1.csv")

# Function to calculate LCA for beams and slabs
LCA.beams_slabs <- function(slab.length, slab.width, beam.length, beam.width, slab.thickness, beam.height, num_beams, beam.Option, slab.Option,
                            materials,dist.Events) {
  # function’s body
  slab.length <- 20  # units: m
  beam.length <- 20  # units: m
  slab.width <- 15   # units: m
  beam.width <- 0.3   # units: m
  slab.thickness <- 0.2  # units: m
  beam.height <- 0.5     # units: m
  num_beams <- 10
  # Split materials based on scope
  materials.split <- split(materials, materials$scope) 
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  # Calculate the volume of the beams and slabs based on different options
  if (beam.Option == "RC") {
    beam.volume <- beam.length * beam.width * beam.height * num_beams * 0
    interventions.beam <- s2$DC + s2$RS + 0.25*s2$G + 0.35(if (!is.null(s2$S)) {s2$S} else {0})
  } else if (beam.Option == "FA RC") {
    beam.volume <- 0.8 * beam.length * beam.width * beam.height * num_beams * 0
    interventions.beam <- 13
  } else if (beam.Option == "GGBS RC") {
    beam.volume <- 0.7 * beam.length * beam.width * beam.height * num_beams
    interventions.beam <- 13
  }
  if (slab.Option == "RC") {
    slab.volume <- slab.length * slab.width * slab.thickness * 0
    interventions.slab <- s2$DC + s2$RS + 0.36*s2$G + 0.54(if (!is.null(s2$S)) {s2$S} else {0})
  } else if (slab.Option == "FA RC") {
    slab.volume <- 0.8 * slab.length * slab.width * slab.thickness * 0
    interventions.slab <- 13
  } else if (slab.Option == "GGBS RC") {
    slab.volume <- 0.7 * slab.length * slab.width * slab.thickness
    interventions.slab <- 13
  }
  
  # Calculate LCA matrix
  beams <- mutate(materials.split[[beam.Option]], villa.Q = beam.volume, interventions =
                    interventions.beam)
  slabs <- mutate(materials.split[[slab.Option]], villa.Q = slab.volume, interventions =
                    interventions.slab)
  
  LCA.matrix <- rbind(beams, slabs)
  
  # Calculate LCA results
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * villa.Q / 1000,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOX.LC = materials.LC * NOX * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC)) 
  
  return(LCA.results) 
}
# Defining main Variables to run the code
# Define building dimensions
slab.length <- 20  # units: m
beam.length <- 20  # units: m
slab.width <- 15   # units: m
beam.width <- 0.3   # units: m
slab.thickness <- 0.2  # units: m
beam.height <- 0.5     # units: m
# Number of beams
num_beams <- 10
# Define the design options for beams and slabs
beam.options <- c("RC", "FA RC", "GGBS RC")
slab.options <- c("RC", "FA RC", "GGBS RC")
# Call the function to calculate the impact on environment
Option.beam_slab <- LCA.beams_slabs(slab.length, beam.length, slab.width, beam.width,
                           slab.thickness, beam.height, num_beams, beam.options[3], slab.options[3], LCI.materials, maintenance.Factory)


#---------------------------Railway system parameters
#Function determining energy consume for our System
LCA.rail <- function(length, thickness, sleeper.Option, trackbed.Option, materials, dist.Events){
  #To make it easier to calculate, we will calculate the energy consume of the system for 1.2m long railway track. The distance between 2 sleepers
  #is around 60cm and the width is 30cm for concrete sleepers and 26cm for wood sleepers. That means for a 1.2m long railway track there will be 2 concrete sleepers or 2.3 wood sleepers.
  
  wood.Section <- 0.0416
  length.reference <- 1.2 # the length of the railway track analyzed
  length.S <- 2.6 # the length for concrete and wood sleepers is the same 2.6m
  prc.Section <- 0.063
  ballast.Section <- 2.08
  steel.rail.unitWeight <- 54.54 #the weight for S54 Steel Rail
  #length.A <- 1.2
  width.A <- 3.2
  asphalt.Q <- length * width.A * thickness
  materials.split <- split(materials, materials$scope)
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  #sleeper options .... We will use a scaling factor so the sleeper volume can be adjusted to be proportional to the actual length of the track
  if (sleeper.Option == "PRC") {
    sleeper.volume <- prc.Section * length.S * 2 * (length / length.reference)
    interventions.sleeper <- s2$DC + (if (!is.null(s2$R)) {s2$R} else {0}) + 0.25*(if (!is.null(s2$RPR)) {s2$RPR} else {0}) + 0.40*(if (!is.null(s2$M)) {s2$M} else {0})
  } else if (sleeper.Option=="wood"){
    sleeper.volume <- wood.Section * 0 * length.S * 2 * (length / length.reference)
    interventions.sleeper <- 2
  } else if (sleeper.Option=="slab") {
    sleeper.volume <- prc.Section * 0 * length.S * 2 * (length / length.reference)
    interventions.sleeper <- 1
  }
  
  
  #track bed options
  if (trackbed.Option =="ballast") {
    trackbed.volume <- ballast.Section * length
    interventions.trackbed <- interventions.sleeper
    #interventions.trackbed <- s2$DC + (if (!is.null(s2$RPR)) {s2$R} else {0}) + 0.10*(if (!is.null(s2$RPR)) {s2$RPR} else {0}) + 0.40*(if (!is.null(s2$M)) {s2$M} else {0})
  } else if (trackbed.Option =="asphalt") {
    trackbed.volume <- asphalt.Q 
    interventions.trackbed <- 2
  }
  
  #steel rail is in every design option
  rail.interventions = length(dist.Events$event)-1
  rail <- mutate(materials.split$steel, railway.Q = steel.rail.unitWeight * length, 
                 interventions = rail.interventions)
  print(materials.split)
  
  
  #We mutate now the different sleeper options
  sleeper <- mutate(materials.split[[sleeper.Option]], railway.Q = sleeper.volume, interventions = interventions.sleeper)
  print(sleeper)
  #We mutate now the different track bed options
  trackbed <- mutate(materials.split[[trackbed.Option]], railway.Q = trackbed.volume, interventions = interventions.trackbed )
  print(trackbed)
  
  #Create LCA Matrix with the different options defined
  LCA.matrix <- rbind (sleeper, trackbed, rail)
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * railway.Q/1000,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOX.LC = materials.LC * NOX * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  
  LCA.results <- list (Energy = sum(LCA.matrix$Energy.LC),
                       CO2 = sum (LCA.matrix$CO2.LC),
                       NOX = sum (LCA.matrix$NOX.LC),
                       SO2 = sum (LCA.matrix$SO2.LC))
  return(LCA.results)
}
#Defining main variables to run the code
# Now we add the variables for 120m Railway track
r.length <- 120 #Unit in m
asphalt.tk <- 0.3 #Unit in m

# Design Options
sleeper.Options <- c("wood", "PRC", "slab")
trackbed.Options <- c("ballast", "asphalt")
# Call the function to calculate the impact on environment
Option.Railway <- LCA.rail(r.length, asphalt.tk, sleeper.Options[2], trackbed.Options[1], LCI.materials, maintenance.Railway_System)


####---------------Purlins parameters
LCA.purlin <- function(total.length, width, purlin.Option, materials, dist.Events ) {
  # Function body
  SEC.WeightperLength <- 13.4  # the weight for UPN120 steel profile
  HRcoil.WeightperLength <- 8.2  # the weight for C channel 200*65*3mm steel profile
  HotDipGalvanized.WeightperLength <- 8.2  # the weight for C channel 200*65*3mm steel profile
  
  materials.split <- split(materials, materials$scope)
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  # purlins options
  # Get the total weight of sections type
  if (purlin.Option == "SEC") {
    # Get the number of purlins
    n <- round(width / 1.5, 0)
    interventions.purlin <- 1.0
    weight <- n * SEC.WeightperLength * total.length
  } else if (purlin.Option == "HR-coil") {
    n <- round(width / 1.5, 0)
    interventions.purlin <- 1.7
    weight <- n * HRcoil.WeightperLength * total.length
  } else if (purlin.Option == "HD-galv") {
    n <- round(width / 1.5, 0)
    interventions.purlin <- s2$DC + s2$TR + 0.55*(if (!is.null(s2$PR)) {s2$PR} else {0}) + 0.15*(if (!is.null(s2$IC)) {s2$IC} else {0})
    weight <- n * HotDipGalvanized.WeightperLength * total.length
  }
  purlin <- mutate(materials.split[[purlin.Option]], total.weight = weight, interventions = interventions.purlin)
  LCA.matrix <- rbind(purlin)
  
  LCA.matrix <- mutate(LCA.matrix,
                       TotalMaterials.Q = quantities * total.weight / 1000,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOX.LC = materials.LC * NOX * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results)
}

total.length <- 80 # units: m
width <- 15 # units: m

purlin.Option <- c("SEC", "HR-coil", "HD-galv")

Option.purlin <- LCA.purlin(total.length, width, purlin.Option[3], LCI.materials, maintenance.Factory)  


#----------------foundation


#Calculation of the total amount of energy consumed for the piles
LCA.pile <- function(diameter, depth, RM.Option, materials, dist.Events) {
  # function’s body
  
pile.Section <- pi * (diameter / 2)^2
pile.volume <- pile.Section * depth
  
#Use the split() function to create subsets of materials based on scopes
materials.split <- split(LCI.materials, LCI.materials$scope)
s <- summary(as.factor(dist.Events$event))
s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))

# Calculate the volume of the pile based on different concrete design mix
if (RM.Option == "RM1") {
  pile.volume <- pile.Section * depth
  interventions.pile <- s2$DC + (if (!is.null(s2$RW)) {s2$RW} else {0}) + 0.5*(if (!is.null(s2$RC)) {s2$RC} else {0}) + 0.25*(if (!is.null(s2$I)) {s2$I} else {0})
} else if (RM.Option == "RM2") {
  pile.volume <- pile.Section * depth * 0.1
  interventions.pile <- 23
} else if (RM.Option == "RM3") {
  pile.volume <- pile.Section * depth * 0.1
  interventions.pile <- 22
  RM.Option <- "RM1" # This line seems out of place, please check if it's necessary here
}

  
# Create matrix with information needed for life cycle analysis
pile <- mutate(materials.split[[RM.Option]], pile.Q = pile.volume, interventions = interventions.pile)
LCA.matrix <- rbind(pile)
# Calculate total amount of materials for the pile elements
LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * pile.Q / 1000,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOX.LC = materials.LC * NOX * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  
# Calculate total amount of energy and emissions for the pile
LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  
# Return LCA results
return(LCA.results)
  
}

# Create vectors for different design options
concrete.Options <- c("RM1", "RM2", "RM3")


# Initialize variables
pile.depth <- 25  # units: m
pile.diameter <- 0.90  # units: m

# Create vectors for different design options
RM.Options <- c("RM1", "RM2", "RM3")

# Get LCA results for different options
Option.pile <- LCA.pile(pile.depth, pile.diameter, RM.Options[1], LCI.materials, maintenance.Factory)


#---------------------------------------------------- LCA Integrated System ------------------------------------------------------

integrated.Design <- as.data.frame(list(Energy = Option.pile$Energy + Option.beam_slab$Energy+ Option.Railway$Energy+ Option.purlin$Energy,
                                        CO2 = Option.pile$CO2 + Option.beam_slab$CO2+ Option.Railway$CO2+ Option.purlin$CO2,
                                        NOX = Option.pile$NOX + Option.beam_slab$NOX+ Option.Railway$NOX+ Option.purlin$NOX,
                                        SO2 = Option.pile$SO2 + Option.beam_slab$SO2+ Option.Railway$SO2+ Option.purlin$SO2))
Energy.costs <- 0.128
CO2.unitcost <- 26 # per metric tone
NOX.unitCost <- 42 # per metric tone
SO2.unitCosts <- 85 # per metric tone

integrated.Design <- mutate(integrated.Design,
                            Costs = (Energy * Energy.costs + 
                                       CO2 * CO2.unitcost +
                                       NOX * NOX.unitCost +
                                       SO2 * SO2.unitCosts)/10^9)



#------------------- Multi-Objective Optimization

#Maintainance Planing Strategies
#Factory 
##


fitness <-
  function(x){
    #define the outputdimension 
    z <- numeric(7)
    #function body
    x <- round(x, 0)
    y <- expand.grid(IC = x[1], PR = x[2], TR = x[3], S = x[4], G = x[5], RS = x[6], 
                     RP = x[7], M = x[8], RPR = x[9], I = x[10], RC = x[11], RW = x[12], R = x[13])
    dur.ev1 <- unlist(y[1:7 / 2])
    dur.ev2 <- unlist(y[8:13/ 2])
    
    dist.1 <-apply(y[1:7], 1, FUN=dist.Events, lifetime=lifetime, start.Date=start.Date)
    dist.2 <-apply(y[8:13], 1, FUN=dist.Events,lifetime=lifetime, start.Date=start.Date)
    results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dur.ev1, dur.ev2)
    z[1] <- sum(results[["duration"]])
    z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
    #Parameters
    #Slab and Beam
    slab.length <- 20  # units: m
    beam.length <- 20  # units: m
    slab.width <- 15   # units: m
    beam.width <- 0.3   # units: m
    slab.thickness <- 0.2  # units: m
    beam.height <- 0.5     # units: m
    num_beams <- 10
    #Railway
    wood.Section <- 0.0416
    length.reference <- 1.2 # the length of the railway track analyzed
    length.S <- 2.6 # the length for concrete and wood sleepers is the same 2.6m
    prc.Section <- 0.063
    ballast.Section <- 2.08
    steel.rail.unitWeight <- 54.54 #the weight for S54 Steel Rail 
    width.A <- 3.2
    asphalt.Q <- r.length * width.A * asphalt.tk
    r.length <- 120 #Unit in m
    asphalt.tk <- 0.3 #Unit in m
    #Purlins
    SEC.WeightperLength <- 13.4  # the weight for UPN120 steel profile
    HRcoil.WeightperLength <- 8.2  # the weight for C channel 200*65*3mm steel profile
    HotDipGalvanized.WeightperLength <- 8.2  # the weight for C channel 200*65*3mm steel profile
    total.length <- 80 # units: m
    width <- 15 # units: m
    #Foundation
    # Initialize variables
    pile.depth <- 25  # units: m
    pile.diameter <- 0.90  # units: m
    pile.Section <- pi * (pile.diameter / 2)^2
    pile.volume <- pile.Section * pile.depth

    #Call the function for LCA using the logic for desing integration
    product.1.Slab_Beam <- LCA.beams_slabs(slab.length, beam.length, slab.width, beam.width,
                                           slab.thickness, beam.height, num_beams, beam.options[3], slab.options[3], LCI.materials, dist.1[[1]])
    product.2.Railway <- LCA.rail(r.length, asphalt.tk, sleeper.Options[2], trackbed.Options[1], LCI.materials, dist.2[[1]])
    product.3.Purlin <- LCA.purlin(total.length, width, purlin.Option[3], LCI.materials,dist.1[[1]])  
    product.4.Foundation <- LCA.pile(pile.depth, pile.diameter, concrete.Options[1], LCI.materials,dist.2[[1]])

    integrated.system <- as.data.frame(list(Energy = product.1.Slab_Beam$Energy + product.2.Railway$Energy + product.3.Purlin$Energy + product.4.Foundation$Energy,
                                            CO2 = product.1.Slab_Beam$CO2 + product.2.Railway$CO2 + product.3.Purlin$CO2 + product.4.Foundation$CO2,
                                            NOX = product.1.Slab_Beam$NOX + product.2.Railway$NOX + product.3.Purlin$NOX + product.4.Foundation$NOX,
                                            SO2 = product.1.Slab_Beam$SO2 + product.2.Railway$SO2 + product.3.Purlin$SO2 + product.4.Foundation$SO2))
    integrated.system <- mutate(integrated.system,
                                Costs = (Energy * Energy.costs +
                                           CO2*CO2.unitcost +
                                           NOX * NOX.unitCost +
                                           SO2 * SO2.unitCosts)/10^9)
    
    z[3] <- integrated.system$Energy
    z[4] <- integrated.system$CO2
    z[5] <- integrated.system$NOX
    z[6] <- integrated.system$SO2
    z[7] <- integrated.system$Costs

    return (z)
  }




#increasing population size to 100 and the number of generations to 10
r2 <- nsga2(fitness, idim = 13, odim = 7,
            generations=10, popsize=100,
            lower.bounds=c(1,36,40,20,10,30,1,1,5,1,45,75,30),
            upper.bounds= c(5,30,60,30,20,40,10,5,15,10,55,85,50))
r2$value
r2$par
r2$pareto.optimal
which(r2$pareto.optimal==TRUE)

#Plotting
r2Results <- as.data.frame(r2$value)
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" )
colnames(r2Results) <- outNames

pareto3 <- as.data.frame(paretoFront(r2))
colnames(pareto3) <- outNames

ggplot(r2Results, aes(x = duration, y = cost)) + geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") +
  geom_line(data = pareto3, color="blue")

input.params <- round(r2$par, 0)
all.results <- cbind(input.params, r2Results, r2$pareto.optimal)
colnames(all.results) <- c("IC","PR","TR","S","G","RS","RP","M","RPR","I","RC","RW","R",
                           "duration", "interv.dist",
                           "energy", "co2", "nox", "so2", "cost",
                           "pareto")
par(mfrow = c(1,1))
parcoord(all.results[, 1:20], var.label = T,
         col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"),
         lty = ifelse(all.results$pareto == TRUE, 1, 3),
         lwd = ifelse(all.results$pareto == TRUE, 3, 1))


